-----------------------------------------------------------------------------------------------
-- Client Lua Script for Narcis
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- Narcis Module Definition
-----------------------------------------------------------------------------------------------
local Narcis = {} 

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Narcis:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

	self.currCostumeId = CostumesLib.GetCostumeIndex()
	
    return o
end

function Narcis:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Narcis OnLoad
-----------------------------------------------------------------------------------------------
function Narcis:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Narcis.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)

	-- Event fired when the costume has been changed
	Apollo.RegisterEventHandler("AppearanceChanged", "OnCostumeUpdate", self)

	-- Register slash commands /narcis stuff
	Apollo.RegisterSlashCommand("narcis", "OnSlashCommand", self)
	
	-- Object containing user options as well as saved costume data
	self.settings = {}
	
	-- Various user options
	self.settings.user = {}	
	self.settings.user.debug = false
	
	-- Saved costume options
	self.settings.costumes = {}
	
	-- Translation table
	self.sideToStr = { [0] = "Left", [1] = "Right", [2] = "Back" }

	-- The current costume ID (0-7, 0 is no costume)
	self.currCostumeId = CostumesLib.GetCostumeIndex()

end

-----------------------------------------------------------------------------------------------
-- Narcis OnDocLoaded
-----------------------------------------------------------------------------------------------
function Narcis:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "NarcisForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- Narcis Data Save/Restore Functions
-----------------------------------------------------------------------------------------------
-- Currently, saving and loading is very basic. Would have to be more interesting if I decide
-- to use a window and everything with options and such... (Populating radios, etc)
function Narcis:OnSave(eLevel)
	if (eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character) then
		return
	end

	-- Do we pass by reference and want to copy it then?
	local saved = self.settings
	
	return saved
end

function Narcis:OnRestore(eLevel, savedData)
	if savedData and savedData.user and savedData.costumes then
		self.settings.user = savedData.user
		self.settings.costumes = savedData.costumes
	end
end

-----------------------------------------------------------------------------------------------
-- Narcis Functions
-----------------------------------------------------------------------------------------------
-- Handle /narcis <command> commands
function Narcis:OnSlashCommand(cmd, params)

    p = string.lower(params)

    if not p or p == "" then
        print_cmd("Narcis by Chimpy on Caretaker")
		print_cmd("--- FOR SCIENCE ---")
		print_cmd("Toggle debug information")
		print_cmd("    /narcis debug")
	elseif p == "debug" then
		self:ToggleDebug()
	end
end

-- Main driver
function Narcis:OnCostumeUpdate()
	local nid = CostumesLib.GetCostumeIndex()
	local oid = self.currCostumeId

	-- Don't try to do anything if we haven't changed costumes
	if nid == oid then
		self:PrintDebug("Appearance changed but costume didn't.")
		return
	end
	
	-- First, the costume ID Change
	print_db("Costume switch: " .. oid .. " -> " .. nid)
	self.currCostumeId = nid

	-- Save the settings for the old costume first
	self:SaveSettings(oid)
	self:PrintDebug("Saving before change...")
	self:PrintCostumeSettings(oid)
	
	-- If we don't have settings for the new costume, copy from the old
	if self.settings.costumes[nid] == nil then
		self:PrintDebug("Copying current settings to new costume...")
		self:SaveSettings(nid)
	end
	
	-- Update the settings to the new id
	self:UpdateSettings(self.settings.costumes[nid])
	self:PrintDebug("Loaded costume is...")
	self:PrintCostumeSettings(nid)
end

-- Saves the current holomark settings to the costume index provided
function Narcis:SaveSettings(idx)
	local new_settings = {}
	
	-- True:  Holomark is visible
	-- False: Holomark is not visible
	local vis_arr = {}
	for i = 0, 2 do
		vis_arr[i] = GameLib.GetGuildHolomarkVisible(i)
	end
	new_settings.visibility = vis_arr

	-- True:  Near
	-- False: Far
	new_settings.distance = GameLib.GetGuildHolomarkDistance()

	self.settings.costumes[idx] = new_settings
end

-- Passes a group of settings to the client
function Narcis:UpdateSettings(settings)
	local lvis = settings.visibility[0]
	local rvis = settings.visibility[1]
	local bvis = settings.visibility[2]
	local dis = settings.distance

	GameLib.ShowGuildHolomark(lvis, rvis, bvis, dis)
end

-----------------------------------------------------------------------------------------------
-- Narcis Debug Functions
-----------------------------------------------------------------------------------------------
-- Event called when "/narcis debug" is used
function Narcis:ToggleDebug()
	if self.settings.user.debug then
		self.settings.user.debug = false
		print_db("Debug turned off")
	else
		self.settings.user.debug = true
		print_db("Debug turned on")
	end
end

-- Conditional print only when debug is enabled
function Narcis:PrintDebug(str)

	if not self.settings.user.debug then
		return
	end

	print_db(str)
end

-- Prints a representation of the costume index argument's settings
function Narcis:PrintCostumeSettings(cid)

	if not self.settings.user.debug then
		return
	end
	
	local settings = self.settings.costumes[cid]
	
	-- First, the costume we're dumping
	print_db("Costume: " .. cid)
	
	-- Print visibility settings
	for i, is_visible in pairs(settings.visibility) do
		local side_str = self.sideToStr[i]
		if is_visible then
			print_db(side_str .. " is visible")
		else
			print_db(side_str .. " is not visible")
		end
	end

	-- Finally, the distance
	if settings.distance then
		print_db("Displayed Near")
	else
		print_db("Displayed Far")
	end
end

-----------------------------------------------------------------------------------------------
-- NarcisForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function Narcis:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function Narcis:OnCancel()
	self.wndMain:Close() -- hide the window
end

-----------------------------------------------------------------------------------------------
-- Local functions
-----------------------------------------------------------------------------------------------
--Helpers used to print to various channels
function print_db(str)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Debug, str)
end

function print_cmd(str)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, str)
end

-----------------------------------------------------------------------------------------------
-- Narcis Instance
-----------------------------------------------------------------------------------------------
local NarcisInst = Narcis:new()
NarcisInst:Init()
